﻿Option Explicit On
Option Strict On

Imports Game_PluginAPI
Imports System.IO
Imports System.IO.MemoryMappedFiles
Imports System.Math
Imports System.Xml.Serialization
Imports System.Runtime.InteropServices

Public Class GamePlugin
    Implements IPlugin_Game
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '///                            SimTools Plugin - Edit the Setting below to provide support for your favorite game!                             ///
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '//////////////////////////////////////////////// 
    '/// Per Game Settings - Change for Each Game ///
    '////////////////////////////////////////////////
    Private Const _PluginAuthorsName As String = "value1, yobuddy"
    Private Const _GameName As String = "Race 07" 'GameName (Must Be Unique!) - the displayed Name.
    Private Const _ProcessName As String = "Race_Steam" 'Process_Name without the (".exe") for this game
    Private Const _Port As String = "4123"  'Your Sending/Recieving UDP Port for this game
    Private Const _RequiresPatchingPath As Boolean = False 'do we need the game exe path for patching? (must be True if _RequiresSecondCheck = True)
    Private Const _RequiresSecondCheck As Boolean = False 'Used when games have the same _ProcessName. (all plugins with the same _ProcessName must use this!)
    Private Const _PluginOptions As String = "" 'Reserved For Future Use - No Change Needed 
    '////////////////////////////////////////////////
    '///           Memory Map Variables           ///
    '////////////////////////////////////////////////
    Private Const _Enable_MemoryMap As Boolean = True 'Is a MemoryMap file Required for this game?
    Private Const _MMF_Name As String = "$Race$" 'Set if using a Memory Map File - EXAMPLE("$gtr2$")
    '////////////////////////////////////////////////
    '///           MemoryHook Variables           ///
    '////////////////////////////////////////////////   
    Private Const _Enable_MemoryHook As Boolean = False 'Is a Memory Hook Required for this game? 
    Private Const _MemHook_Roll As UInteger = 0 'Not Used = 0
    Private Const _MemHook_Pitch As UInteger = 0
    Private Const _MemHook_Heave As UInteger = 0
    Private Const _MemHook_Yaw As UInteger = 0
    Private Const _MemHook_Sway As UInteger = 0
    Private Const _MemHook_Surge As UInteger = 0
    Private Const _MemHook_Extra1 As UInteger = 0
    Private Const _MemHook_Extra2 As UInteger = 0
    Private Const _MemHook_Extra3 As UInteger = 0
    '////////////////////////////////////////////////
    '///    DOFs Used for Output for this Game    ///
    '////////////////////////////////////////////////
    Private Const _DOF_Support_Roll As Boolean = True
    Private Const _DOF_Support_Pitch As Boolean = True
    Private Const _DOF_Support_Heave As Boolean = True
    Private Const _DOF_Support_Yaw As Boolean = True
    Private Const _DOF_Support_Sway As Boolean = True
    Private Const _DOF_Support_Surge As Boolean = True
    Private Const _DOF_Support_Extra1 As String = "" 'Blank = False
    Private Const _DOF_Support_Extra2 As String = "" '"" = Not Used
    Private Const _DOF_Support_Extra3 As String = "" 'ADD THE FORCE NAME HERE
    '/////////////////////////////////////////////////
    '///       GameDash - Dash Board Support       ///
    '/////////////////////////////////////////////////
    Private Const _Enable_DashBoard As Boolean = True  'Enable the DashBoard Output System?
    'EXAMPLES OF DASH OUTPUT (all variables are strings) - 20 DASH OUTPUTS MAX!!!
    'Use the Variables with either Process_PacketRecieved, Process_MemoryHook or Process_MemoryMap.
    'Variable = (*Action, Value) as String - (The *Action variable can be anything)
    'Basic Dash Support should include at the minimum -  SPEED, RPM and GEAR
    'Dash_1_Output = "Speed," & OUTPUT_VALUE_HERE.ToString
    'Dash_2_Output = "Rpm," & OUTPUT_VALUE_HERE.ToString
    'Dash_2_Output = "Gear," & OUTPUT_VALUE_HERE.ToString
    '...
    'Dash_20_Output = "Engine Temp," & OUTPUT_VALUE_HERE.ToString
    '/////////////////////////////////////////////////
    '///           GameVibe Support              ///
    '/////////////////////////////////////////////////
    Private Const _Enable_GameVibe As Boolean = False 'Enable the GameVibe Output System?
    'EXAMPLES OF GameVibe OUTPUT (all variables are strings) - 9 GameVibe OUTPUTS MAX!!!
    'Use the Variables with either Process_PacketRecieved, Process_MemoryHook or Process_MemoryMap.
    'Variable = (*Action, Value) as String - (The *Action" variable can ONLY be: Rpm, Heave, Sway, Surge, FR, FL, RR, RL, Turbo)
    'Basic GameVibe Support should include at the minimum -  RPM
    'Vibe_1_Output = "Rpm," & OUTPUT_VALUE_HERE.ToString
    'Vibe_2_Output = "Heave," & OUTPUT_VALUE_HERE.ToString
    'Vibe_3_Output = "Sway," & OUTPUT_VALUE_HERE.ToString
    'Vibe_4_Output = "Surge," & OUTPUT_VALUE_HERE.ToString
    '...
    'Vibe_9_Output = "Gear Shift," & OUTPUT_VALUE_HERE.ToString
    '/////////////////////////////////////////////////

    'Used by GameManager when the Game Starts.
    Public Sub GameStart() Implements IPlugin_Game.GameStart
        'DO SOMETHING HERE AT GAME START!
    End Sub

    'Used by GameManager when the Game Stops.
    Public Sub GameStop() Implements IPlugin_Game.GameEnd
        'DO SOMETHING HERE AT GAME STOP!
    End Sub

    'Used by GameManager to Process a MemoryHook.
    Public Sub Process_MemoryHook() Implements IPlugin_Game.Process_MemoryHook
        'DO SOMETHING HERE AT GAME START!        
    End Sub

    'Used by GameManager to Process a MemoryMap.
    Public Sub Process_MemoryMap() Implements IPlugin_Game.Process_MemoryMap
         Dim RadToDeg As Double = 45.0 / Atan(1.0)
        Try
            Using file = MemoryMappedFile.OpenExisting(_MMF_Name)
                Using reader = file.CreateViewAccessor(0, Marshal.SizeOf(MMFSimBin))
                    Dim bytes = New Byte(Marshal.SizeOf(MMFSimBin)) {}
                    reader.ReadArray(Of Byte)(0, bytes, 0, bytes.Length)
                    Dim ByteArray() As Byte = bytes
                    'Create Gchandle instance and pin variable required
                    Dim MyGC As GCHandle = GCHandle.Alloc(MMFSimBin, GCHandleType.Pinned)
                    'get address of variable in pointer variable
                    Dim AddofLongValue As IntPtr = MyGC.AddrOfPinnedObject()
                    'Copy the memory space to my GCHandle
                    Marshal.Copy(ByteArray, 0, AddofLongValue, ByteArray.Length)
                    'Direct Cast myGC to my Outsim Object
                    MMFSimBin = DirectCast(MyGC.Target, Struct_MMFSimBin)
                    'Free GChandle to avoid memory leaks
                    MyGC.Free()

                    'assign values - clip very small values
                    Roll_MemMap = MMFSimBin.Roll * RadToDeg
                    Pitch_MemMap = -MMFSimBin.Pitch * RadToDeg
                    Heave_MemMap = MMFSimBin.Heave
                    Yaw_MemMap = -MMFSimBin.Yaw * RadToDeg
                    Sway_MemMap = -MMFSimBin.Sway
                    Surge_MemMap = -MMFSimBin.Surge
                    Dash_1_Output = "Speed," & MMFSimBin.ActualVel * 3.6
                    Dash_2_Output = "RPM," & MMFSimBin.ActualEngineRPM
                    Dash_3_Output = "Gear," & MMFSimBin.ActualGear
                    Dash_4_Output = "Actual Fuel Pressure [kPa]," & MMFSimBin.ActualFuelPressure
                    Dash_5_Output = "Actual Fuel Liters," & MMFSimBin.ActualFuelLiters
                    Dash_6_Output = "Fuel Capacity [l]," & MMFSimBin.FuelCapacityLiters
                    Dash_7_Output = "Actual Engine Water Temp [°C]," & MMFSimBin.ActualEngineWaterTemp
                    Dash_8_Output = "Actual Engine OilTemp [°C]," & MMFSimBin.ActualEngineOilTemp
                    Dash_9_Output = "Actual Engine Oil Pressure [kPa]," & MMFSimBin.ActualEngineOilPressure
                    Dash_10_Output = "Lap," & MMFSimBin.completedLaps & "of " & MMFSimBin.numberOfLaps
                    Dash_11_Output = "Best Lap Time [sec]," & MMFSimBin.lapTimeBest
                    Dash_12_Output = "Previous Lap [sec]," & MMFSimBin.lapTimePrevious
                    Dash_13_Output = "Current Lap Time [sec]," & MMFSimBin.lapTimeCurrent
                    Dash_14_Output = "Position," & MMFSimBin.position
                    Dash_15_Output = "# of Cars in Race," & MMFSimBin.numCars
                End Using
            End Using
        Catch ex As Exception
        End Try
    End Sub

    'Used by GameEngine to Process Incoming UDP Packets.
    Public Sub Process_PacketRecieved(Text As String) Implements IPlugin_Game.Process_PacketRecieved
         'DO SOMETHING HERE TO RECIEVED UDP PACKETS
        Input_Data = CType(DeSerializeString(Text), GenericOutput)
        'Get Proper Data out of UDP Packet
        With Input_Data
            Roll_Output = (._Roll)
            Pitch_Output = (._Pitch)
            Heave_Output = (._Heave)
            Yaw_Output = (._Yaw)
            Sway_Output = (._Sway)
            Surge_Output = (._Surge)
        End With
    End Sub

    'Used by GameManager to Patch a Game.
    Public Function PatchGame(ByVal MyPath As String, ByVal MyIp As String) As Boolean Implements IPlugin_Game.PatchGame
        'Change as Needed
        MsgBox("Patch Installed!", MsgBoxStyle.OkOnly, "Patching info")
        Return True
    End Function

    'Used by GameManager to UnPatch a Game.
    Public Sub UnPatchGame(MyPath As String) Implements IPlugin_Game.UnPatchGame
        'Change as Needed
        UnPatch(MyPath)
        MsgBox("Patch Uninstalled!", MsgBoxStyle.OkOnly, "Patching info")
    End Sub

    'Used by GameManager to UnPatch a Game.
    Private Sub UnPatch(MyPath As String)
        'Change as Needed
    End Sub

    'Tells the User where to patch the game 
    Public Sub PatchPathInfo() Implements IPlugin_Game.PatchPathInfo
        'Tell the User where to patch the game 
        'Change as Needed
    End Sub

    'Used by GameManager to Validate a Path befors Patching.
    Public Function ValidatePatchPath(MyPath As String) As Boolean Implements IPlugin_Game.ValidatePatchPath
        'insert a simple validation of the patching path - let the user know he got it right
        'Change as Needed
    End Function


    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '///                                                 PLACE EXTRA NEEDED CODE/FUNCTIONS HERE                                                     ///
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#Region "///// - EXTRA CODE/FUNCTIONS USED FOR THIS PLUGIN - /////"
    Private MMFSimBin As Struct_MMFSimBin
    Private Input_Data As New GenericOutput

    'Generic In/Output structure
    Public Structure GenericOutput
        Public _Roll As Double
        Public _Pitch As Double
        Public _Heave As Double
        Public _Yaw As Double
        Public _Sway As Double
        Public _Surge As Double
        Public _Extra1 As Double
        Public _Extra2 As Double
        Public _Extra3 As Double
    End Structure

    'DeSerialize
    Public Function DeSerializeString(InputString As String) As Object
        On Error Resume Next
        Dim xml_serializer As New XmlSerializer(GetType(GenericOutput))
        Dim string_reader As New StringReader(InputString)
        Dim DeserializedOutput As GenericOutput = DirectCast(xml_serializer.Deserialize(string_reader), GenericOutput)
        string_reader.Close()
        Return DeserializedOutput
    End Function

    'required for SimBin Games
    Private Structure Struct_MMFSimBin
        Public userInputs1 As Single
        Public userInputs2 As Single
        Public userInputs3 As Single
        Public userInputs4 As Single
        Public userInputs5 As Single
        Public userInputs6 As Single
        'This structure allows for a number of parameters to be
        'passed from the user to the exteral application via control input
        'in the game. The ISIInputType enum describes which element of this
        'array corresponds to which in-game control. Note that this data
        'is floating point, and can be configured in the game to be driven
        'by an analog input device (joystick). The user may also map the
        'control to a keyboard key, or a digital controller button. This
        'means that the value may be anywhere between 0.0 and 1.0. Also note
        'that these values will not be debounced; if the user
        'maps the "External Signal Up" control to a key on the keyboard,
        'the coresponding value in this array will remain 1.0 for as long
        'as the user holds the key down.

        Public ActualEngineRPM As Single 'Engine speed, Radians Per Second.
        Public EngineMaxRPM As Single 'For use with an "analog" rpm display.
        Public ActualFuelPressure As Single 'KPa
        Public ActualFuelLiters As Single 'Current liters of fuel in the tank(s).
        Public FuelCapacityLiters As Single 'Maximum capacity of fuel tank(s).
        Public ActualEngineWaterTemp As Single
        Public ActualEngineOilTemp As Single
        Public ActualEngineOilPressure As Single

        Public ActualVel As Single 'meters per second
        Public numberOfLaps As Int32 '# of laps in race, or -1 if player is not in
        '                             race mode (player is in practice or test mode).
        Public completedLaps As Int32 'How many laps the player has completed. If this
        '                              value is 6, the player is on his 7th lap. -1 = n/a
        Public lapTimeBest As Single 'Seconds. -1.0 = none
        Public lapTimePrevious As Single 'Seconds. -1.0 = none
        Public lapTimeCurrent As Single 'Seconds. -1.0 = none
        Public position As Int32 'Current position. 1 = first place.
        Public numCars As Int32 'Number of cars (including the player) in the race.
        Public ActualGear As Int32 '-2 = no data available, -1 = reverse, 0 = neutral,
        '                           1 = first gear... (valid range -1 to 7).

        'tireTemp: Temperature of three points across the tread of each tire.
        Public tireTempFL_L As Single
        Public tireTempFL_M As Single
        Public tireTempFL_R As Single
        Public tireTempFR_L As Single
        Public tireTempFR_M As Single
        Public tireTempFR_R As Single
        Public tireTempRL_L As Single
        Public tireTempRL_M As Single
        Public tireTempRL_R As Single
        Public tireTempRR_L As Single
        Public tireTempRR_M As Single
        Public tireTempRR_R As Single

        Public numPenalties As Int32

        Public X_WC As Single 'Physical location of car's Center of Gravity in world space, X,Y,Z... Y=up.
        Public Y_WC As Single
        Public Z_WC As Single

        Public Pitch As Single  ' in GTR2 this actually is X_WC instead
        Public Yaw As Single    ' in GTR2 this actually is Y_WC instead
        Public Roll As Single   ' in GTR2 this actually is Z_WC instead
        'local acceleration in three axes (X, Y, Z) of car body (divide by
        '9.81 to get G-force). From car center, +X=left, +Y=up, +Z=back.
        Public Sway As Single ' lateral force left-right
        Public Heave As Single 'vertical force up-down
        Public Surge As Single 'longitudinal force faster, slower
    End Structure
#End Region


    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '///                                                        DO NOT EDIT BELOW HERE!!!                                                           ///
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#Region "///// BUILT IN METHODS FOR SIMTOOLS - DO NOT CHANGE /////"
    'Output Dash Vars
    Public Dash_1_Output As String = ""
    Public Dash_2_Output As String = ""
    Public Dash_3_Output As String = ""
    Public Dash_4_Output As String = ""
    Public Dash_5_Output As String = ""
    Public Dash_6_Output As String = ""
    Public Dash_7_Output As String = ""
    Public Dash_8_Output As String = ""
    Public Dash_9_Output As String = ""
    Public Dash_10_Output As String = ""
    Public Dash_11_Output As String = ""
    Public Dash_12_Output As String = ""
    Public Dash_13_Output As String = ""
    Public Dash_14_Output As String = ""
    Public Dash_15_Output As String = ""
    Public Dash_16_Output As String = ""
    Public Dash_17_Output As String = ""
    Public Dash_18_Output As String = ""
    Public Dash_19_Output As String = ""
    Public Dash_20_Output As String = ""

    'Output Vars
    Public Roll_Output As Double = 0
    Public Pitch_Output As Double = 0
    Public Heave_Output As Double = 0
    Public Yaw_Output As Double = 0
    Public Sway_Output As Double = 0
    Public Surge_Output As Double = 0
    Public Extra1_Output As Double = 0
    Public Extra2_Output As Double = 0
    Public Extra3_Output As Double = 0

    'MemHook Vars
    Public Roll_MemHook As Double = 0
    Public Pitch_MemHook As Double = 0
    Public Heave_MemHook As Double = 0
    Public Yaw_MemHook As Double = 0
    Public Sway_MemHook As Double = 0
    Public Surge_MemHook As Double = 0
    Public Extra1_MemHook As Double = 0
    Public Extra2_MemHook As Double = 0
    Public Extra3_MemHook As Double = 0

    'MemMap Vars
    Public Roll_MemMap As Double = 0
    Public Pitch_MemMap As Double = 0
    Public Heave_MemMap As Double = 0
    Public Yaw_MemMap As Double = 0
    Public Sway_MemMap As Double = 0
    Public Surge_MemMap As Double = 0
    Public Extra1_MemMap As Double = 0
    Public Extra2_MemMap As Double = 0
    Public Extra3_MemMap As Double = 0

    'GameVibe Vars
    Public Vibe_1_Output As String = ""
    Public Vibe_2_Output As String = ""
    Public Vibe_3_Output As String = ""
    Public Vibe_4_Output As String = ""
    Public Vibe_5_Output As String = ""
    Public Vibe_6_Output As String = ""
    Public Vibe_7_Output As String = ""
    Public Vibe_8_Output As String = ""
    Public Vibe_9_Output As String = ""

    Public Function Get_PluginVersion() As String Implements IPlugin_Game.Get_PluginVersion
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Version.ToString
    End Function

    Public Function GetDOFsUsed() As String Implements IPlugin_Game.GetDOFsUsed
        'Return DOF's Used (Roll,Pitch,Heave,Yaw,Sway,Surge)
        Return (_DOF_Support_Roll.ToString & "," & _DOF_Support_Pitch.ToString & "," & _DOF_Support_Heave.ToString & "," & _DOF_Support_Yaw.ToString & "," & _DOF_Support_Sway.ToString & "," & _DOF_Support_Surge.ToString & "," & _DOF_Support_Extra1.ToString & "," & _DOF_Support_Extra2.ToString & "," & _DOF_Support_Extra3.ToString)
    End Function

    Public Sub ResetDOFVars() Implements IPlugin_Game.ResetDOFVars
        Roll_Output = 0
        Pitch_Output = 0
        Heave_Output = 0
        Yaw_Output = 0
        Sway_Output = 0
        Surge_Output = 0
        Extra1_Output = 0
        Extra2_Output = 0
        Extra3_Output = 0
    End Sub

    Public Sub ResetMapVars() Implements IPlugin_Game.ResetMapVars
        Roll_MemMap = 0
        Pitch_MemMap = 0
        Heave_MemMap = 0
        Yaw_MemMap = 0
        Sway_MemMap = 0
        Surge_MemMap = 0
        Extra1_MemMap = 0
        Extra2_MemMap = 0
        Extra3_MemMap = 0
    End Sub

    Public Sub ResetHookVars() Implements IPlugin_Game.ResetHookVars
        Roll_MemHook = 0
        Pitch_MemHook = 0
        Heave_MemHook = 0
        Yaw_MemHook = 0
        Sway_MemHook = 0
        Surge_MemHook = 0
        Extra1_MemHook = 0
        Extra2_MemHook = 0
        Extra3_MemHook = 0
    End Sub

    Public Sub ResetDashVars() Implements IPlugin_Game.ResetDashVars
        Dash_1_Output = ""
        Dash_2_Output = ""
        Dash_3_Output = ""
        Dash_4_Output = ""
        Dash_5_Output = ""
        Dash_6_Output = ""
        Dash_7_Output = ""
        Dash_8_Output = ""
        Dash_9_Output = ""
        Dash_10_Output = ""
        Dash_11_Output = ""
        Dash_12_Output = ""
        Dash_13_Output = ""
        Dash_14_Output = ""
        Dash_15_Output = ""
        Dash_16_Output = ""
        Dash_17_Output = ""
        Dash_18_Output = ""
        Dash_19_Output = ""
        Dash_20_Output = ""
    End Sub

    Public Sub ResetVibeVars() Implements IPlugin_Game.ResetVibeVars
        Vibe_1_Output = ""
        Vibe_2_Output = ""
        Vibe_3_Output = ""
        Vibe_4_Output = ""
        Vibe_5_Output = ""
        Vibe_6_Output = ""
        Vibe_7_Output = ""
        Vibe_8_Output = ""
        Vibe_9_Output = ""
    End Sub

    Public ReadOnly Property PluginAuthorsName() As String Implements IPlugin_Game.PluginAuthorsName
        Get
            Return _PluginAuthorsName
        End Get
    End Property

    Public ReadOnly Property Name() As String Implements IPlugin_Game.GameName
        Get
            Return _GameName
        End Get
    End Property

    Public ReadOnly Property ProcessName() As String Implements IPlugin_Game.ProcessName
        Get
            Return _ProcessName
        End Get
    End Property

    Public ReadOnly Property Port() As String Implements IPlugin_Game.Port
        Get
            Return _Port
        End Get
    End Property

    Public ReadOnly Property Enable_MemoryMap() As Boolean Implements IPlugin_Game.Enable_MemoryMap
        Get
            Return _Enable_MemoryMap
        End Get
    End Property

    Public ReadOnly Property Enable_MemoryHook() As Boolean Implements IPlugin_Game.Enable_MemoryHook
        Get
            Return _Enable_MemoryHook
        End Get
    End Property

    Public ReadOnly Property RequiresPatchingPath() As Boolean Implements IPlugin_Game.RequiresPatchingPath
        Get
            Return _RequiresPatchingPath
        End Get
    End Property

    Public ReadOnly Property RequiresSecondCheck() As Boolean Implements IPlugin_Game.RequiresSecondCheck
        Get
            Return _RequiresSecondCheck
        End Get
    End Property

    Public ReadOnly Property Enable_DashBoard() As Boolean Implements IPlugin_Game.Enable_DashBoard
        Get
            Return _Enable_DashBoard
        End Get
    End Property

    Public ReadOnly Property Enable_GameVibe() As Boolean Implements IPlugin_Game.Enable_GameVibe
        Get
            Return _Enable_GameVibe
        End Get
    End Property

    Public Function Get_RollOutput() As Double Implements IPlugin_Game.Get_RollOutput
        Return Roll_Output
    End Function

    Public Function Get_PitchOutput() As Double Implements IPlugin_Game.Get_PitchOutput
        Return Pitch_Output
    End Function

    Public Function Get_HeaveOutput() As Double Implements IPlugin_Game.Get_HeaveOutput
        Return Heave_Output
    End Function

    Public Function Get_YawOutput() As Double Implements IPlugin_Game.Get_YawOutput
        Return Yaw_Output
    End Function

    Public Function Get_SwayOutput() As Double Implements IPlugin_Game.Get_SwayOutput
        Return Sway_Output
    End Function

    Public Function Get_SurgeOutput() As Double Implements IPlugin_Game.Get_SurgeOutput
        Return Surge_Output
    End Function

    Public Function Get_Extra1Output() As Double Implements IPlugin_Game.Get_Extra1Output
        Return Extra1_Output
    End Function

    Public Function Get_Extra2Output() As Double Implements IPlugin_Game.Get_Extra2Output
        Return Extra2_Output
    End Function

    Public Function Get_Extra3Output() As Double Implements IPlugin_Game.Get_Extra3Output
        Return Extra3_Output
    End Function

    Public Function Get_Dash_1_Output() As String Implements IPlugin_Game.Get_Dash1_Output
        Return Dash_1_Output
    End Function

    Public Function Get_Dash_2_Output() As String Implements IPlugin_Game.Get_Dash2_Output
        Return Dash_2_Output
    End Function

    Public Function Get_Dash_3_Output() As String Implements IPlugin_Game.Get_Dash3_Output
        Return Dash_3_Output
    End Function

    Public Function Get_Dash_4_Output() As String Implements IPlugin_Game.Get_Dash4_Output
        Return Dash_4_Output
    End Function

    Public Function Get_Dash_5_Output() As String Implements IPlugin_Game.Get_Dash5_Output
        Return Dash_5_Output
    End Function

    Public Function Get_Dash_6_Output() As String Implements IPlugin_Game.Get_Dash6_Output
        Return Dash_6_Output
    End Function

    Public Function Get_Dash_7_Output() As String Implements IPlugin_Game.Get_Dash7_Output
        Return Dash_7_Output
    End Function

    Public Function Get_Dash_8_Output() As String Implements IPlugin_Game.Get_Dash8_Output
        Return Dash_8_Output
    End Function

    Public Function Get_Dash_9_Output() As String Implements IPlugin_Game.Get_Dash9_Output
        Return Dash_9_Output
    End Function

    Public Function Get_Dash_10_Output() As String Implements IPlugin_Game.Get_Dash10_Output
        Return Dash_10_Output
    End Function

    Public Function Get_Dash_11_Output() As String Implements IPlugin_Game.Get_Dash11_Output
        Return Dash_11_Output
    End Function

    Public Function Get_Dash_12_Output() As String Implements IPlugin_Game.Get_Dash12_Output
        Return Dash_12_Output
    End Function

    Public Function Get_Dash_13_Output() As String Implements IPlugin_Game.Get_Dash13_Output
        Return Dash_13_Output
    End Function

    Public Function Get_Dash_14_Output() As String Implements IPlugin_Game.Get_Dash14_Output
        Return Dash_14_Output
    End Function

    Public Function Get_Dash_15_Output() As String Implements IPlugin_Game.Get_Dash15_Output
        Return Dash_15_Output
    End Function

    Public Function Get_Dash_16_Output() As String Implements IPlugin_Game.Get_Dash16_Output
        Return Dash_16_Output
    End Function

    Public Function Get_Dash_17_Output() As String Implements IPlugin_Game.Get_Dash17_Output
        Return Dash_17_Output
    End Function

    Public Function Get_Dash_18_Output() As String Implements IPlugin_Game.Get_Dash18_Output
        Return Dash_18_Output
    End Function

    Public Function Get_Dash_19_Output() As String Implements IPlugin_Game.Get_Dash19_Output
        Return Dash_19_Output
    End Function

    Public Function Get_Dash_20_Output() As String Implements IPlugin_Game.Get_Dash20_Output
        Return Dash_20_Output
    End Function

    Public Function Get_RollMemHook() As Double Implements IPlugin_Game.Get_RollMemHook
        Return Roll_MemHook
    End Function

    Public Function Get_PitchMemHook() As Double Implements IPlugin_Game.Get_PitchMemHook
        Return Pitch_MemHook
    End Function

    Public Function Get_HeaveMemHook() As Double Implements IPlugin_Game.Get_HeaveMemHook
        Return Heave_MemHook
    End Function

    Public Function Get_YawMemHook() As Double Implements IPlugin_Game.Get_YawMemHook
        Return Yaw_MemHook
    End Function

    Public Function Get_SwayMemHook() As Double Implements IPlugin_Game.Get_SwayMemHook
        Return Sway_MemHook
    End Function

    Public Function Get_SurgeMemHook() As Double Implements IPlugin_Game.Get_SurgeMemHook
        Return Surge_MemHook
    End Function

    Public Function Get_Extra1MemHook() As Double Implements IPlugin_Game.Get_Extra1MemHook
        Return Extra1_MemHook
    End Function

    Public Function Get_Extra2MemHook() As Double Implements IPlugin_Game.Get_Extra2MemHook
        Return Extra2_MemHook
    End Function

    Public Function Get_Extra3MemHook() As Double Implements IPlugin_Game.Get_Extra3MemHook
        Return Extra3_MemHook
    End Function

    Public Function Get_RollMemMap() As Double Implements IPlugin_Game.Get_RollMemMap
        Return Roll_MemMap
    End Function

    Public Function Get_PitchMemMap() As Double Implements IPlugin_Game.Get_PitchMemMap
        Return Pitch_MemMap
    End Function

    Public Function Get_HeaveMemMap() As Double Implements IPlugin_Game.Get_HeaveMemMap
        Return Heave_MemMap
    End Function

    Public Function Get_YawMemMap() As Double Implements IPlugin_Game.Get_YawMemMap
        Return Yaw_MemMap
    End Function

    Public Function Get_SwayMemMap() As Double Implements IPlugin_Game.Get_SwayMemMap
        Return Sway_MemMap
    End Function

    Public Function Get_SurgeMemMap() As Double Implements IPlugin_Game.Get_SurgeMemMap
        Return Surge_MemMap
    End Function

    Public Function Get_Extra1MemMap() As Double Implements IPlugin_Game.Get_Extra1MemMap
        Return Extra1_MemMap
    End Function

    Public Function Get_Extra2MemMap() As Double Implements IPlugin_Game.Get_Extra2MemMap
        Return Extra2_MemMap
    End Function

    Public Function Get_Extra3MemMap() As Double Implements IPlugin_Game.Get_Extra3MemMap
        Return Extra3_MemMap
    End Function

    Public Function Get_Vibe1_Output() As String Implements IPlugin_Game.Get_Vibe1_Output
        Return Vibe_1_Output
    End Function

    Public Function Get_Vibe2_Output() As String Implements IPlugin_Game.Get_Vibe2_Output
        Return Vibe_2_Output
    End Function

    Public Function Get_Vibe3_Output() As String Implements IPlugin_Game.Get_Vibe3_Output
        Return Vibe_3_Output
    End Function

    Public Function Get_Vibe4_Output() As String Implements IPlugin_Game.Get_Vibe4_Output
        Return Vibe_4_Output
    End Function

    Public Function Get_Vibe5_Output() As String Implements IPlugin_Game.Get_Vibe5_Output
        Return Vibe_5_Output
    End Function

    Public Function Get_Vibe6_Output() As String Implements IPlugin_Game.Get_Vibe6_Output
        Return Vibe_6_Output
    End Function

    Public Function Get_Vibe7_Output() As String Implements IPlugin_Game.Get_Vibe7_Output
        Return Vibe_7_Output
    End Function

    Public Function Get_Vibe8_Output() As String Implements IPlugin_Game.Get_Vibe8_Output
        Return Vibe_8_Output
    End Function

    Public Function Get_Vibe9_Output() As String Implements IPlugin_Game.Get_Vibe9_Output
        Return Vibe_9_Output
    End Function

    Public ReadOnly Property PluginOptions() As String Implements IPlugin_Game.PluginOptions
        Get
            Return _PluginOptions
        End Get
    End Property
#End Region
End Class